module Manager.ViewManager exposing (documentView)
-- Browser
import Browser
-- Html
import Html            exposing (Html, Attribute, div, text)
import Html.Attributes exposing (class, style, href, rel, id, title, value, src)
-- Own
import TypeHolder exposing (Model, Msg(..))
import Manager.StateManager exposing (..)
import Manager.ModalManager exposing (..)
import Util.ViewParts    exposing (..)
import Util.ShapeManager exposing (..)
import Util.Util         exposing (..)

documentView : Model -> Browser.Document Msg
documentView model = if model.edit
                       then { title = "ShapePicture_Editor"
                            , body = [viewEditor model]
                            }
                       else { title = "ShapePicture_Viewer"
                            , body = [viewViewer model]
                            }

viewEditor : Model -> Html Msg
viewEditor model =
  div [style "background-color" "#fff"]
      (List.concat
        [ standardViewPart
        ,[ viewHeader_Buttons model.state
         , Html.section [class "section"] []
         , viewImage model
         , case model.shapesLoadResult of
             Ok  s -> text ""
             Err e -> text ("Could not load shapes: "++(httpErrorToString e))
         ]
        , case (getModalFromState model.state) of
            NoModal                -> []
            ShowInput iShape ix iy -> [viewNewShapeInputModal iShape ix iy ]
            ShowTable              -> [viewTableModal         model.saved  ]
            ShowShape id edit      -> [viewShapeModal         id edit
                                          ( model.saved
                                              |> List.take (id  )
                                              |> List.drop (id-1)
                                              |> List.head
                                              |> Maybe.withDefault (Empty,defaultShapeStyle)
                                              |> Tuple.second
                                          )
                                      ]
            ShowJson                    -> [viewJsonModal           model                      ]
            EditShape id shapeStyle     -> [viewEditShapeModal      id shapeStyle              ]
            ShowManagePicture s url w h -> [viewManagePictureModal  s url model.picture.url w h]
        ]
      )

viewViewer : Model -> Html Msg
viewViewer model =
  div [style "background-color" "#fff"]
      (List.concat
        [ standardViewPart
        ,[viewImage model]
        ]
      )